@@ -51,18 +51,21 @@ wchar_t *unix_name(doscp_t *fromDos,
 		   const char *base, const char *ext, uint8_t Case,
 		   wchar_t *answer);
 
+/* Is this a "special" directory entry, such as . or .. ? */
+int isSpecial(const char *name);
+
 #ifdef HAVE_WCHAR_H
 int isSpecialW(const wchar_t *name);
 #else
 #define isSpecialW isSpecial
 #endif
 
-UNUSED(static __inline__ wchar_t ch_towupper(wchar_t ch))
+static inline wchar_t ch_towupper(wchar_t ch)
 {
         return (wchar_t) towupper( (wint_t) ch);
 }
 
-UNUSED(static __inline__ wchar_t ch_towlower(wchar_t ch))
+static inline wchar_t ch_towlower(wchar_t ch)
 {
         return (wchar_t) towlower( (wint_t) ch);
 }
