@@ -26,16 +26,11 @@
 #include "sysincludes.h"
 #include "stream.h"
 #include "mtools.h"
-#include "msdos.h"
 #include "open_image.h"
 #include "devices.h"
 #include "plain_io.h"
 #include "llong.h"
 
-#ifdef HAVE_LINUX_FS_H
-# include <linux/fs.h>
-#endif
-
 typedef struct SimpleFile_t {
     struct Stream_t head;
 
@@ -47,6 +42,7 @@ typedef struct SimpleFile_t {
 #ifdef OS_hpux
     int size_limited;
 #endif
+    const char *postcmd;
 } SimpleFile_t;
 
 
@@ -91,7 +87,7 @@ static ssize_t file_io(SimpleFile_t *This, char *buf,
 #endif
 
 	if ( ret == -1 ){
-		perror("plain_io");
+		perror("plain_io read/write");
 		return -1;
 	}
 	This->lastwhere = where + ret;
@@ -138,9 +134,11 @@ static int file_free(Stream_t *Stream)
 {
 	DeclareThis(SimpleFile_t);
 
-	if (This->fd > 2)
-		return close(This->fd);
-	else
+	if (This->fd > 2) {
+		int ret = close(This->fd);
+		postcmd(This->postcmd);
+		return ret;
+	} else
 		return 0;
 }
 
@@ -339,6 +337,8 @@ APIRET rc;
 	}
 
 	precmd(dev);
+	if(dev)
+	    This->postcmd=dev->postcmd;
 	if(IS_PRIVILEGED(dev) && !(mode2 & NO_PRIV))
 		reclaim_privs();
 
@@ -445,6 +445,7 @@ APIRET rc;
 	return &This->head;
  exit_0:
 	close(This->fd);
+	postcmd(This->postcmd);
  exit_1:
 	Free(This);
 	return NULL;
