@@ -55,7 +55,7 @@ struct vfat_subentry {
 struct vfat_state {
 	wchar_t name[VBUFSIZE];
 	int status; /* is now a bit map of 32 bits */
-	int subentries;
+	unsigned int subentries;
 	unsigned char sum; /* no need to remember the sum for each entry,
 			    * it is the same anyways */
 	int present;
@@ -182,7 +182,7 @@ static int unicode_write(wchar_t *in, unsigned char *out, int num, int *end_p)
 }
 
 
-static __inline__ int unicode_read(unsigned char *in,
+static inline int unicode_read(unsigned char *in,
 				   wchar_t *out, int num)
 {
 	wchar_t *end_out = out+num;
@@ -228,7 +228,7 @@ static void clear_vfat(struct vfat_state *v)
  *
  * David C. Niemi (niemi@tuxers.net) 95.01.19
  */
-static __inline__ unsigned char sum_shortname(const dos_name_t *dn)
+static inline unsigned char sum_shortname(const dos_name_t *dn)
 {
 	unsigned char sum;
 	const char *name=dn->base;
@@ -246,7 +246,7 @@ static __inline__ unsigned char sum_shortname(const dos_name_t *dn)
  * Return 1 if the VSEs comprise a valid long file name,
  * 0 if not.
  */
-static __inline__ void check_vfat(struct vfat_state *v, struct directory *dir)
+static inline void check_vfat(struct vfat_state *v, struct directory *dir)
 {
 	dos_name_t dn;
 
@@ -271,51 +271,9 @@ static __inline__ void check_vfat(struct vfat_state *v, struct directory *dir)
 	v->present = 1;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wsign-conversion"
-/* We have indeed different types for the entry slot
- * - the higher levels have a "signed" type, in order to accomodate
- *   reserved values for "root directory" entry, "not found" entries, and
- *   "uninitialized"
- * - the lower levels always consider it as an index into the
- *   directory viewed as a table, i.e. always positive
- */
-int clear_vses(Stream_t *Dir, int entrySlot, unsigned int last)
-{
-	direntry_t entry;
-	dirCache_t *cache;
-	int error;
-
-	entry.Dir = Dir;
-	entry.entry = entrySlot;
-
-	/*maximize(last, entry.entry + MAX_VFAT_SUBENTRIES);*/
-	cache = allocDirCache(Dir, last);
-	if(!cache) {
-		fprintf(stderr, "Out of memory error in clear_vses\n");
-		exit(1);
-	}
-	addFreeEntry(cache, entry.entry, last);
-	for (; entry.entry < (signed int) last; ++entry.entry) {
-#ifdef DEBUG
-		fprintf(stderr,"Clearing entry %d.\n", entry.entry);
-#endif
-		dir_read(&entry, &error);
-		if(error)
-			return error;
-		if(!entry.dir.name[0] || entry.dir.name[0] == DELMARK)
-			break;
-		entry.dir.name[0] = DELMARK;
-		if (entry.dir.attr == 0xf)
-			entry.dir.attr = '\0';
-		low_level_dir_write(&entry);
-	}
-	return 0;
-}
-
-int write_vfat(Stream_t *Dir, dos_name_t *shortname, char *longname,
-	       unsigned int start,
-	       direntry_t *mainEntry)
+unsigned int write_vfat(Stream_t *Dir, dos_name_t *shortname, char *longname,
+			unsigned int start,
+			direntry_t *mainEntry)
 {
 	struct vfat_subentry *vse;
 	uint8_t vse_id, num_vses;
@@ -363,7 +321,7 @@ int write_vfat(Stream_t *Dir, dos_name_t *shortname, char *longname,
 			       start + num_vses - vse_id, start + num_vses);
 #endif
 
-			entry.entry = start + num_vses - vse_id;
+			setEntryToPos(&entry, start + num_vses - vse_id);
 			low_level_dir_write(&entry);
 		}
 	} else {
@@ -387,12 +345,12 @@ void dir_write(direntry_t *entry)
 	dirCacheEntry_t *dce;
 	dirCache_t *cache;
 
-	if(entry->entry == -3) {
+	if(isRootEntry(entry)) {
 		fprintf(stderr, "Attempt to write root directory pointer\n");
 		exit(1);
 	}
 
-	cache = allocDirCache(entry->Dir, entry->entry + 1);
+	cache = allocDirCache(entry->Dir, getNextEntryAsPos(entry));
 	if(!cache) {
 		fprintf(stderr, "Out of memory error in dir_write\n");
 		exit(1);
@@ -413,7 +371,7 @@ void dir_write(direntry_t *entry)
  * The following function translates a series of vfat_subentries into
  * data suitable for a dircache entry
  */
-static __inline__ void parse_vses(direntry_t *entry,
+static inline void parse_vses(direntry_t *entry,
 				  struct vfat_state *v)
 {
 	struct vfat_subentry *vse;
@@ -492,7 +450,7 @@ static dirCacheEntry_t *vfat_lookup_loop_common(doscp_t *cp,
 						int *io_error)
 {
 	wchar_t newfile[13];
-	unsigned int initpos = direntry->entry + 1;
+	unsigned int initpos = getNextEntryAsPos(direntry);
 	struct vfat_state vfat;
 	wchar_t *longname;
 	int error;
@@ -508,9 +466,10 @@ static dirCacheEntry_t *vfat_lookup_loop_common(doscp_t *cp,
 			    *io_error = error;
 			    return NULL;
 			}
-			addFreeEndEntry(cache, initpos, direntry->entry,
+			addFreeEndEntry(cache, initpos,
+					getEntryAsPos(direntry),
 					endmarkSeen);
-			return addEndEntry(cache, direntry->entry);
+			return addEndEntry(cache, getEntryAsPos(direntry));
 		}
 
 		if (endmarkSeen || direntry->dir.name[0] == ENDMARK){
@@ -519,7 +478,7 @@ static dirCacheEntry_t *vfat_lookup_loop_common(doscp_t *cp,
 				endmarkSeen = 1;
 				continue;
 			}
-			return addEndEntry(cache, direntry->entry);
+			return addEndEntry(cache, getEntryAsPos(direntry));
 		}
 		if(direntry->dir.name[0] != DELMARK &&
 		   direntry->dir.attr == 0x0f)
@@ -536,7 +495,7 @@ static dirCacheEntry_t *vfat_lookup_loop_common(doscp_t *cp,
 	/* deleted file */
 	if (direntry->dir.name[0] == DELMARK) {
 		return addFreeEntry(cache, initpos,
-				    direntry->entry + 1);
+				    getNextEntryAsPos(direntry));
 	}
 
 	check_vfat(&vfat, &direntry->dir);
@@ -545,7 +504,7 @@ static dirCacheEntry_t *vfat_lookup_loop_common(doscp_t *cp,
 
 	/* mark space between last entry and this one as free */
 	addFreeEntry(cache, initpos,
-		     direntry->entry - vfat.subentries);
+		     getEntryAsPos(direntry) - vfat.subentries);
 
 	if (direntry->dir.attr & 0x8){
 		/* Read entry as a label */
@@ -570,12 +529,12 @@ static dirCacheEntry_t *vfat_lookup_loop_common(doscp_t *cp,
 	else
 		longname = 0;
 
-	return addUsedEntry(cache, direntry->entry - vfat.subentries,
-			    direntry->entry + 1, longname,
+	return addUsedEntry(cache, getEntryAsPos(direntry) - vfat.subentries,
+			    getNextEntryAsPos(direntry), longname,
 			    newfile, &direntry->dir);
 }
 
-static __inline__ dirCacheEntry_t *vfat_lookup_loop_for_read(doscp_t *cp,
+static inline dirCacheEntry_t *vfat_lookup_loop_for_read(doscp_t *cp,
 							     direntry_t *direntry,
 							     dirCache_t *cache,
 							     int *io_error)
@@ -586,7 +545,7 @@ static __inline__ dirCacheEntry_t *vfat_lookup_loop_for_read(doscp_t *cp,
 	*io_error = 0;
 	dce = cache->entries[initpos];
 	if(dce) {
-		direntry->entry = dce->endSlot - 1;
+		setEntryToPos(direntry, dce->endSlot - 1);
 		return dce;
 	} else {
 		return vfat_lookup_loop_common(cp,
@@ -705,10 +664,10 @@ int vfat_lookup(direntry_t *direntry, const char *filename,
 	else
 		length = 0;
 
-	if (direntry->entry == -2)
+	if (isNotFound(direntry))
 		return -1;
 
-	cache = allocDirCache(direntry->Dir, direntry->entry+1);
+	cache = allocDirCache(direntry->Dir, getNextEntryAsPos(direntry));
 	if(!cache) {
 		fprintf(stderr, "Out of memory error in vfat_lookup [0]\n");
 		exit(1);
@@ -742,15 +701,15 @@ int vfat_lookup(direntry_t *direntry, const char *filename,
 		direntry->endSlot = dce->endSlot-1;
 		return 0; /* file found */
 	} else {
-		direntry->entry = -2;
+		direntry->entry = NOT_FOUND_ENTRY;
 		return -1; /* no file found */
 	}
 }
 
-static __inline__ dirCacheEntry_t *vfat_lookup_loop_for_insert(doscp_t *cp,
-							       direntry_t *direntry,
-							       unsigned int initpos,
-							       dirCache_t *cache)
+static inline dirCacheEntry_t *vfat_lookup_loop_for_insert(doscp_t *cp,
+							   direntry_t *direntry,
+							   unsigned int initpos,
+							   dirCache_t *cache)
 {
 	dirCacheEntry_t *dce;
 	int io_error;
@@ -759,7 +718,7 @@ static __inline__ dirCacheEntry_t *vfat_lookup_loop_for_insert(doscp_t *cp,
 	if(dce && dce->type != DCET_END) {
 		return dce;
 	} else {
-		direntry->entry = initpos - 1;
+		setEntryForIteration(direntry, initpos);
 		dce = vfat_lookup_loop_common(cp,
 					      direntry, cache, 1, &io_error);
 		if(!dce) {
@@ -793,7 +752,7 @@ static void accountFreeSlots(struct scan_state *ssp, dirCacheEntry_t *dce)
 static void clear_scan(wchar_t *longname, int use_longname,
 		       struct scan_state *s)
 {
-	s->shortmatch = s->longmatch = s->slot = -1;
+	s->shortmatch = s->longmatch = -1;
 	s->free_end = s->got_slots = s->free_start = 0;
 
 	if (use_longname & 1)
@@ -882,7 +841,8 @@ int lookupForInsert(Stream_t *Dir,
 				    !wcscasecmp(dce->longName, wlongname)) ||
 				   (dce->shortName &&
 				    !wcscasecmp(dce->shortName, wlongname))) {
-					ssp->longmatch = dce->endSlot - 1;
+					ssp->longmatch =
+						(int) (dce->endSlot - 1);
 					/* long match is a reason for
 					 * immediate stop */
 					direntry->beginSlot = dce->beginSlot;
@@ -894,7 +854,8 @@ int lookupForInsert(Stream_t *Dir,
 				 * short name match */
 				if (!ignore_match &&
 				    !wcscasecmp(shortName, dce->shortName))
-					ssp->shortmatch = dce->endSlot - 1;
+					ssp->shortmatch =
+						(int) (dce->endSlot - 1);
 				break;
 			case DCET_END:
 				break;
@@ -914,7 +875,6 @@ int lookupForInsert(Stream_t *Dir,
 	fprintf(stderr, "No directory slots\n");
 	return -1;
 }
-#pragma GCC diagnostic pop
 
 
 /* End vfat.c */
